﻿using Microsoft.Owin.Security;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Sample.Models;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Security.Claims;
using System.Text;
using System.Web;
using System.Web.Mvc;

namespace Sample.Controllers
{
    public class AccountsController : Controller
    {
        public static OAuthProperties OAuthProperties
        {
            get;
            set;
        }

        public async System.Threading.Tasks.Task<ActionResult> GenerateToken()
        {
            OAuthProperties = new OAuthProperties();
            var accessToken = string.Empty;
            var uri = new Uri(OAuthProperties.AuthorizationURL);
            var client = new HttpClient();
            var result = new HttpResponseMessage();
            var getCodeUrl = new Uri(OAuthProperties.AuthorizationURL + "?response_type=code&client_id=" + OAuthProperties.ClientId + "&redirect_uri=" + new Uri(Request.Url.AbsoluteUri).GetLeftPart(UriPartial.Authority));
            var code = HttpUtility.ParseQueryString(HttpContext.Request.Url.Query).Get("code");          
         
            if (code != null)
            {
                var getAccessTokenUrl = new Uri(OAuthProperties.TokenURL);
                HttpRequestMessage tokenRequest = new HttpRequestMessage(HttpMethod.Post, getAccessTokenUrl);
                var keyValues = new List<KeyValuePair<string, string>>();
                keyValues.Add(new KeyValuePair<string, string>("grant_type", "authorization_code"));
                keyValues.Add(new KeyValuePair<string, string>("client_id", OAuthProperties.ClientId));
                keyValues.Add(new KeyValuePair<string, string>("redirect_uri", new Uri(Request.Url.AbsoluteUri).GetLeftPart(UriPartial.Authority)));
                keyValues.Add(new KeyValuePair<string, string>("client_secret", OAuthProperties.ClientSecret));
                keyValues.Add(new KeyValuePair<string, string>("code", code));
                HttpContent httpContent = new FormUrlEncodedContent(keyValues);
                tokenRequest.Content = httpContent;
                httpContent = new FormUrlEncodedContent(keyValues);             
                result = await client.PostAsync(getAccessTokenUrl.ToString(), httpContent);        
                var tokenResult = result.Content.ReadAsStringAsync().Result;
                if (tokenResult != null)
                {
                    var token = JObject.Parse(tokenResult);
                    ViewBag.AccessToken = token["access_token"].ToString();                    
                    var tokenValues = Unprotect(token["access_token"].ToString());
                    var uniqueName = tokenValues.Identity.Claims.Where(c => c.Type == JwtClaims.UniqueName).Select(c => c.Value).FirstOrDefault();
                    ViewBag.UniqueName = uniqueName;
                }                
            }
            else
            {
                return Redirect(getCodeUrl.AbsoluteUri);              
            }

            return View("login");
        }

        public AuthenticationTicket Unprotect(string signedAndEncodedToken)
        {
            ClaimsIdentity idenity = null;
            var properties = new AuthenticationProperties();

            var tokenHandler = new JwtSecurityTokenHandler();

            var tokenInfo = tokenHandler.ReadToken(signedAndEncodedToken) as JwtSecurityToken;

            if (tokenInfo != null)
            {
                idenity = new ClaimsIdentity(tokenInfo.Claims);
                properties = new AuthenticationProperties();
            }

            return new AuthenticationTicket(idenity, new AuthenticationProperties());
        }
    }

    public class CustomWebClient : WebClient
    {
        internal CustomWebClient()
        {
            ServicePointManager.ServerCertificateValidationCallback = SuppressCertificateErrors;
        }

        protected override WebRequest GetWebRequest(Uri uri)
        {
            var request = base.GetWebRequest(uri);
            request.Timeout = 4 * 60 * 1000; //Increase time out
            return request;
        }

        private bool SuppressCertificateErrors(object sender, System.Security.Cryptography.X509Certificates.X509Certificate certificate,
           System.Security.Cryptography.X509Certificates.X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            return true;
        }
    }  

}
